Package: dbscan
Version: 1.1-3
Date: 2018-11-12
Title: Density Based Clustering of Applications with Noise (DBSCAN) and
        Related Algorithms
Authors@R: c(person("Michael", "Hahsler", role = c("aut", "cre", "cph"),
                email = "mhahsler@lyle.smu.edu"),
	    person("Matthew", "Piekenbrock", role = c("aut", "cph")),
	    person("Sunil", "Arya", role = c("ctb", "cph")),
	    person("David", "Mount", role = c("ctb", "cph")))
Description: A fast reimplementation of several density-based algorithms of
    the DBSCAN family for spatial data. Includes the DBSCAN (density-based spatial
    clustering of applications with noise) and OPTICS (ordering points to identify
    the clustering structure) clustering algorithms HDBSCAN (hierarchical DBSCAN) and the LOF (local outlier
    factor) algorithm. The implementations use the kd-tree data structure (from
    library ANN) for faster k-nearest neighbor search. An R interface to fast kNN
    and fixed-radius NN search is also provided.
Imports: Rcpp (>= 0.12.12), graphics, stats, methods
LinkingTo: Rcpp
Suggests: fpc, microbenchmark, testthat, dendextend, igraph, knitr,
        DMwR
VignetteBuilder: knitr
BugReports: https://github.com/mhahsler/dbscan
License: GPL (>= 2)
Copyright: ANN library is copyright by University of Maryland, Sunil
        Arya and David Mount. All other code is copyright by Michael
        Hahsler and Matthew Piekenbrock.
SystemRequirements: C++11
NeedsCompilation: yes
Packaged: 2018-11-13 19:37:41 UTC; hahsler
Author: Michael Hahsler [aut, cre, cph],
  Matthew Piekenbrock [aut, cph],
  Sunil Arya [ctb, cph],
  David Mount [ctb, cph]
Maintainer: Michael Hahsler <mhahsler@lyle.smu.edu>
Repository: CRAN
Date/Publication: 2018-11-13 22:50:48 UTC
Built: R 3.5.1; x86_64-w64-mingw32; 2019-05-30 19:59:59 UTC; windows
Archs: x64
