# Generated by roxygen2: do not edit by hand

S3method(adjacencylist,NN)
S3method(adjacencylist,frNN)
S3method(adjacencylist,kNN)
S3method(as.dendrogram,default)
S3method(as.dendrogram,hclust)
S3method(as.dendrogram,hdbscan)
S3method(as.dendrogram,optics)
S3method(as.dendrogram,reachability)
S3method(as.reachability,dendrogram)
S3method(as.reachability,optics)
S3method(augment,dbscan)
S3method(augment,general_clustering)
S3method(augment,hdbscan)
S3method(comps,dist)
S3method(comps,frNN)
S3method(comps,kNN)
S3method(comps,sNN)
S3method(glance,dbscan)
S3method(glance,general_clustering)
S3method(glance,hdbscan)
S3method(nobs,dbscan)
S3method(nobs,general_clustering)
S3method(nobs,hdbscan)
S3method(plot,NN)
S3method(plot,hdbscan)
S3method(plot,optics)
S3method(plot,reachability)
S3method(predict,dbscan_fast)
S3method(predict,hdbscan)
S3method(predict,optics)
S3method(print,dbscan_fast)
S3method(print,frNN)
S3method(print,general_clustering)
S3method(print,hdbscan)
S3method(print,kNN)
S3method(print,optics)
S3method(print,reachability)
S3method(print,sNN)
S3method(sort,NN)
S3method(sort,frNN)
S3method(sort,kNN)
S3method(sort,sNN)
S3method(tidy,dbscan)
S3method(tidy,general_clustering)
S3method(tidy,hdbscan)
export(adjacencylist)
export(as.dendrogram)
export(as.reachability)
export(augment)
export(comps)
export(coredist)
export(dbscan)
export(extractDBSCAN)
export(extractFOSC)
export(extractXi)
export(frNN)
export(glance)
export(glosh)
export(hdbscan)
export(hullplot)
export(is.corepoint)
export(jpclust)
export(kNN)
export(kNNdist)
export(kNNdistplot)
export(lof)
export(mrdist)
export(optics)
export(pointdensity)
export(sNN)
export(sNNclust)
export(tidy)
import(Rcpp)
importFrom(generics,augment)
importFrom(generics,glance)
importFrom(generics,tidy)
importFrom(grDevices,adjustcolor)
importFrom(grDevices,chull)
importFrom(grDevices,palette)
importFrom(graphics,abline)
importFrom(graphics,lines)
importFrom(graphics,matplot)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,points)
importFrom(graphics,polygon)
importFrom(graphics,segments)
importFrom(graphics,text)
importFrom(stats,as.dendrogram)
importFrom(stats,dendrapply)
importFrom(stats,dist)
importFrom(stats,hclust)
importFrom(stats,is.leaf)
importFrom(stats,nobs)
importFrom(stats,prcomp)
importFrom(stats,predict)
importFrom(utils,tail)
useDynLib(dbscan, .registration=TRUE)
